from datetime import datetime
from formencode.validators import FancyValidator, Invalid

class NaiveDateTimeValidator(FancyValidator):

    def _to_python(self, value, status):
        date_string = value.strip()
        try:
            date = datetime.strptime(date_string, '%Y-%m-%d %H:%M')
        except:
            raise Invalid('date must be in format yyyy-mm-dd hh:mm', value, status)
        return date



# -*- coding: utf-8 -*-
from sqlalchemy.ext.declarative import declarative_base
from tgext.pluggable import PluggableSession, app_model
import tgext.tagging

DBSession = PluggableSession()
DeclarativeBase = declarative_base()

def init_model(app_session):
    DBSession.configure(app_session)

from models import Article, Attachment, Blog




# -*- coding: utf-8 -*-
"""Main Controller"""
from tg import TGController
from tg import expose, flash, require, url, lurl, request, redirect, validate, tmpl_context, config
from tg.decorators import before_render, cached_property, paginate
from tg.i18n import ugettext as _, lazy_ugettext as l_

try:
    from tg import predicates
except ImportError:
    from repoze.what import predicates

from tgext.pluggable import app_model
from smallpress.model import DBSession, Article, Attachment, Blog
from smallpress.helpers import *
from smallpress.lib.forms import UploadForm, get_article_form, DataGrid, \
    CSSLink, inject_datagrid_resources, SpinnerIcon
from tgext.datahelpers.validators import SQLAEntityConverter
from tgext.datahelpers.fields import AttachedFile
from tgext.pluggable import plug_url, plug_redirect
from webhelpers.html.builder import HTML
from formencode.validators import UnicodeString
from datetime import datetime
from tgext.ajaxforms.ajaxform import formexpose
from tgext.tagging.controllers import TaggingController
from tgext.crud import EasyCrudRestController

AttachmentType = config.get('_smallpress',{}).get('attachment_type', AttachedFile)

try:
    import whoosh
    from whoosh.query import *
    whoosh_enabled = True
except ImportError:
    whoosh_enabled = False

upload_form = UploadForm()
articles_table = DataGrid(fields=[(l_('Actions'), lambda row:link_buttons(row, 'edit', 'delete', 'hide', 'publish')),
                                  (l_('Blog'), lambda row:row.blog and row.blog.name),
                                  (l_('Title'), lambda row:HTML.a(row.title,
                                                                  href=plug_url('smallpress', '/view/%s'%row.uid,
                                                                                lazy=True))),
                                  (l_('Tags'), comma_separated_tags),
                                  (l_('Author'), 'author'),
                                  (l_('Publishing'), lambda row:format_published(row) + ', ' + format_date(row))])

attachments_table = DataGrid(fields=[(l_('Actions'), lambda row:link_buttons(row, 'rmattachment')),
                                     (l_('Name'), 'name'),
                                     (l_('Preview'), attachment_preview),
                                     (l_('Url'), lambda row:format_link(row.url))])

def inject_css(*args, **kw):
    CSSLink(link='/_pluggable/smallpress/css/style.css').inject()

class BlogsController(EasyCrudRestController):
    allow_only = predicates.in_group('smallpress')
    title = "Manage Blogs"
    model = Blog

    __form_options__ = {
        '__hide_fields__' : ['uid'],
        '__omit_fields__' : ['articles']
    }

    __table_options__ = {
        '__omit_fields__' : ['uid', 'articles']
    }

class RootController(TGController):
    tagging = TaggingController(model=Article, session=DBSession, allow_edit=None)
    tagging.search = before_render(inject_css)(tagging.search)

    @cached_property
    def blogs(self):
        return BlogsController(DBSession.wrapped_session)

    @expose('genshi:smallpress.templates.index')
    def index(self, blog='', *args, **kw):
        articles = Article.get_published(blog).all()
        tags = app_model.Tagging.tag_cloud_for_set(Article, articles).all()
        return dict(articles=articles, tags=tags, blog=blog)

    @expose('genshi:smallpress.templates.article')
    @validate(dict(article=SQLAEntityConverter(Article)), error_handler=index)
    def view(self, article, **kw):
        visible = False

        if article.published and article.publish_date <= datetime.now():
            visible = True
        elif request.identity and article.author == request.identity['user']:
            visible = True
        elif request.identity and 'smallpress' in request.identity['groups']:
            visible = True

        if not visible:
            return redirect(plug_url('smallpress', '/'))

        hooks = config['hooks'].get('smallpress.before_view_article', [])
        for func in hooks:
            func(article, kw)

        return dict(article=article,
                    tg_cache=article.caching_options)

    @require(predicates.in_group('smallpress'))
    @expose('genshi:smallpress.templates.manage')
    @paginate('articles')
    def manage(self, blog='', *args, **kw):
        articles = Article.get_all(blog)
        return dict(table=articles_table, articles=articles,
                    create_action=self.mount_point+'/new/'+blog)

    @require(predicates.in_group('smallpress'))
    @expose('genshi:smallpress.templates.edit')
    def new(self, blog=None, **kw):
        inject_datagrid_resources(attachments_table)

        if 'uid' not in kw:
            if blog:
                blog = DBSession.query(Blog).filter_by(name=blog).first()
            article = Article(author=request.identity['user'], blog=blog and blog)

            hooks = config['hooks'].get('smallpress.before_create_article', [])
            for func in hooks:
                func(article, kw)

            DBSession.add(article)
            DBSession.flush()

            hooks = config['hooks'].get('smallpress.after_create_article', [])
            for func in hooks:
                func(article, kw)
        else:
            article = DBSession.query(Article).get(kw['uid'])

        return plug_redirect('smallpress', '/edit/%s' % article.uid)

    @require(predicates.in_group('smallpress'))
    @expose('genshi:smallpress.templates.edit')
    def edit(self, uid, *args, **kw):
        inject_datagrid_resources(attachments_table)

        article = DBSession.query(Article).get(uid)
        value = {
            'uid':article.uid,
            'title':article.title,
            'description':article.description,
            'tags':comma_separated_tags(article),
            'publish_date':format_date(article),
            'content':article.content
        }

        hooks = config['hooks'].get('smallpress.before_edit_article', [])
        for func in hooks:
            func(article, value)

        return dict(article=article, value=value, blog=article.blog and article.blog.name or '',
                    form=get_article_form(), action=url(self.mount_point+'/save'),
                    upload_form=upload_form, upload_action=url(self.mount_point+'/attach'))

    @require(predicates.in_group('smallpress'))
    @validate(get_article_form(), error_handler=edit)
    @expose()
    def save(self, *args, **kw):
        article = DBSession.query(Article).get(kw['uid'])

        hooks = config['hooks'].get('smallpress.before_save_article', [])
        for func in hooks:
            func(article, kw)

        article.title = kw['title']
        article.description = kw['description']
        article.content = kw['content']
        article.publish_date = kw['publish_date']
        article.update_date = datetime.now()
        app_model.Tagging.set_tags(article, kw['tags'])

        flash(_('Articles successfully saved'))
        return redirect(self.mount_point+'/manage/'+article.blog_name)

    @require(predicates.in_group('smallpress'))
    @formexpose(upload_form, 'smallpress.templates.attachments')
    def upload_form_show(self, **kw):
        article = DBSession.query(Article).get(kw['article'])
        return dict(value=kw, table=attachments_table, attachments=article.attachments)

    @require(predicates.in_group('smallpress'))
    @validate(upload_form, error_handler=upload_form_show)
    @expose('genshi:smallpress.templates.attachments')
    def attach(self, **kw):
        article = DBSession.query(Article).get(kw['article'])
        attachment = Attachment(name=kw['name'], article=article,
                                content=AttachmentType(kw['file'].file, kw['file'].filename))
        DBSession.add(attachment)
        DBSession.flush()

        return dict(value=dict(article=article.uid),
                    ajaxform=upload_form,
                    ajaxform_id='attachments_form',
                    ajaxform_action=upload_form.action,
                    ajaxform_spinner=SpinnerIcon(),
                    table=attachments_table,
                    attachments=article.attachments)

    @require(predicates.in_group('smallpress'))
    @validate(dict(attachment=SQLAEntityConverter(Attachment)))
    @expose()
    def rmattachment(self, attachment):
        article = attachment.article
        DBSession.delete(attachment)
        flash(_('Attachment successfully removed'))
        return redirect(self.mount_point+'/edit/%s'%article.uid)

    @require(predicates.in_group('smallpress'))
    @validate(dict(article=SQLAEntityConverter(Article)), error_handler=manage)
    @expose()
    def publish(self, article):
        if not article.content:
            flash(_('Cannot publish an empty article'))
        else:
            article.published=True
            flash(_('Article published'))
        return redirect(self.mount_point+'/manage/'+article.blog_name)

    @require(predicates.in_group('smallpress'))
    @validate(dict(article=SQLAEntityConverter(Article)), error_handler=manage)
    @expose()
    def hide(self, article):
        article.published=False
        flash(_('Article hidden'))
        return redirect(self.mount_point+'/manage/'+article.blog_name)

    @require(predicates.in_group('smallpress'))
    @validate(dict(article=SQLAEntityConverter(Article)), error_handler=manage)
    @expose()
    def delete(self, article):
        DBSession.delete(article)
        flash(_('Article successfully removed'))
        return redirect(self.mount_point+'/manage/'+article.blog_name)

    @expose('genshi:smallpress.templates.index')
    @validate(dict(text=UnicodeString(not_empty=True)), error_handler=index)
    def search(self, text=None):
        articles = []

        if whoosh_enabled:
            index_path = config.get('smallpress_whoosh_index', '/tmp/smallpress_whoosh')
            ix = whoosh.index.open_dir(index_path)
            with ix.searcher() as searcher:
                query = Or([Term("content", text),
                            Term("title", text),
                            Term("description", text)])
                found = searcher.search(query)
                if len(found):
                    articles = Article.get_published().filter(Article.uid.in_([e['uid'] for e in found])).all()
        else:
            articles = Article.get_published().filter(Article.content.like('%'+text+'%')).all()

        tags = app_model.Tagging.tag_cloud_for_set(Article).all()
        return dict(articles=articles, tags=tags, blog='')



# -*- coding: utf-8 -*-
"""The tgapp-smallpress package"""
import tg, os, logging
from tgext.pluggable import plugged, plug

log = logging.getLogger('tgapp-smallpress')

def plugme(app_config, options):
    tg.config['_smallpress'] = options

    if 'tgext.tagging' not in plugged():
        plug(app_config, 'tgext.tagging')

    def init_whoosh(app):
        try:
            from whoosh.index import create_in
            log.info('Enabling Whoosh Support')
            whoosh_enabled = True
        except ImportError:
            log.info('Whoosh Not Found, disabled')
            whoosh_enabled = False

        if whoosh_enabled:
            from smallpress.model.models import WHOOSH_SCHEMA
            index_path = tg.config.get('smallpress_whoosh_index', '/tmp/smallpress_whoosh')
            if not os.path.exists(index_path):
                os.mkdir(index_path)
                ix = create_in(index_path, WHOOSH_SCHEMA)

        return app

    app_config.register_hook('before_config', init_whoosh)
    return dict(appid='smallpress', global_helpers=False)



# -*- coding: utf-8 -*-
"""Setup the smallpress application"""

from smallpress import model
from tgext.pluggable import app_model
import tg

def bootstrap(command, conf, vars):
    print 'Bootstrapping smallpress...'

    g = app_model.Group(group_name='smallpress', display_name='Smallpress blogging users')
    model.DBSession.add(g)
    model.DBSession.flush()

    u1 = model.DBSession.query(app_model.User).filter_by(user_name='manager').first()
    if u1:
        g.users.append(u1)
    model.DBSession.flush()


import sys
import tg
from tg.i18n import lazy_ugettext as l_
from tgext.ajaxforms import ajaxloaded
from tgext.pluggable import plug_url
from smallpress.lib.validators import NaiveDateTimeValidator

class SpinnerIcon(object):
    @property
    def link(self):
        return tg.url('/_pluggable/smallpress/images/spinner.gif')

if tg.config.get('prefer_toscawidgets2', False):
    from tw2.core import CSSLink, Required
    from tw2.forms import DataGrid
    from tw2.forms import ListForm, TextField, TextArea, HiddenField, FileField, SubmitButton
    from formencode.validators import UnicodeString, FieldStorageUploadConverter

    def inject_datagrid_resources(dg):
        resources = [r.req() for r in dg.resources]
        for r in resources:
            r.prepare()

    class ArticleForm(ListForm):
        uid = HiddenField()
        title = TextField(label='Title', validator=UnicodeString(not_empty=True, outputEncoding=None))
        description = TextField(label='Description', validator=UnicodeString(outputEncoding=None),
                                placeholder=l_('If empty will be extracted from the content'))
        tags = TextField(label='Tags', validator=UnicodeString(outputEncoding=None),
                         placeholder=l_('tags, comma separated'))
        content = TextArea(label=None, key='content', name='content', id="article_content",
                           validator=UnicodeString(not_empty=True, outputEncoding=None))
        publish_date = TextField(label='Publish Date', validator=NaiveDateTimeValidator(not_empty=True))

    @ajaxloaded
    class UploadForm(ListForm):
        article = HiddenField()
        name = TextField(label='Name', validator=UnicodeString(not_empty=True, outputEncoding=None))
        file = FileField(label='File', validator=FieldStorageUploadConverter(not_empty=True))

        action = plug_url('smallpress', '/attach', lazy=True)
        ajaxurl = plug_url('smallpress', '/upload_form_show', lazy=True)

        submit = SubmitButton(value='Attach')

    class SearchForm(ListForm):
        text = TextField(label=None, validator=UnicodeString(not_empty=True, outputEncoding=None),
                         placeholder=l_('Search...'))

        submit = SubmitButton(value='Search')

else:
    from tw.api import CSSLink
    from tw.forms import ListForm, TextField, TextArea, HiddenField, FileField
    from tw.forms import DataGrid
    from tw.core import WidgetsList
    from tw.forms.validators import UnicodeString, FieldStorageUploadConverter

    def inject_datagrid_resources(dg):
        dg.register_resources()

    class ArticleForm(ListForm):
        class fields(WidgetsList):
            uid = HiddenField()
            title = TextField(label_text='Title', validator=UnicodeString(not_empty=True))
            description = TextField(label_text='Description', validator=UnicodeString(),
                                    attrs=dict(placeholder=l_('If empty will be extracted from the content')))
            tags = TextField(label_text='Tags', validator=UnicodeString(),
                             attrs=dict(placeholder=l_('tags, comma separated')))
            content = TextArea(suppress_label=True, validator=UnicodeString(not_empty=True),
                               attrs=dict(id='article_content'))
            publish_date = TextField(label_text='Publish Date', validator=NaiveDateTimeValidator(not_empty=True))

    @ajaxloaded
    class UploadForm(ListForm):
        class fields(WidgetsList):
            article = HiddenField()
            name = TextField(label_text='Name', validator=UnicodeString(not_empty=True))
            file = FileField(label_text='File', validator=FieldStorageUploadConverter(not_empty=True))

        action = plug_url('smallpress', '/attach', lazy=True)
        ajaxurl = plug_url('smallpress', '/upload_form_show', lazy=True)
        submit_text = 'Attach'

    class SearchForm(ListForm):
        class fields(WidgetsList):
            text = TextField(suppress_label=True, validator=UnicodeString(not_empty=True),
                             attrs=dict(placeholder=l_('Search...')))

        submit_text = 'Search'

def get_article_form():
    config = tg.config['_smallpress']

    article_form = config.get('form_instance')
    if not article_form:
        form_path = config.get('form', 'smallpress.lib.forms.ArticleForm')
        module, form_name = form_path.rsplit('.', 1)
        module = __import__(module, fromlist=form_name)
        form_class = getattr(module, form_name)
        article_form = config['form_instance'] = form_class()

    return article_form



from tg import expose, request
from smallpress.model import Article
from tgext.pluggable import plug_url, primary_key, app_model
from tgext.tagging.widgets import TagCloud
from smallpress.lib.forms import SearchForm

search_form = SearchForm()

WEEK_EXPIRE = 3600*24*7

@expose('genshi:smallpress.templates.articles')
def articles(articles=None, blog=None):
    if articles is None:
        articles=Article.get_published(blog)
    return dict(articles=articles)

@expose('genshi:smallpress.templates.article_preview')
def article_preview(article):
    user = 'anonymous'
    if request.identity:
        user = getattr(request.identity['user'], primary_key(app_model.User).key)
    return dict(article=article, tg_cache=dict(key='%s-%s-%s' % (article.uid, user,
                                                                 article.update_date.strftime('%Y-%m-%d_%H:%M:%S')),
                                               expire=WEEK_EXPIRE,
                                               type='memory'))

@expose('genshi:smallpress.templates.tagcloud')
def tagcloud(tags=None):
    tagcloud=TagCloud(tagging_url=plug_url('smallpress', '/tagging'))
    if tags is None:
        tags = app_model.Tagging.tag_cloud_for_set(Article).all()
    return dict(tagcloud=tagcloud, tags=tags)

@expose('genshi:smallpress.templates.search')
def search():
    return dict(form=search_form, action=plug_url('smallpress', '/search'),
                tg_cache=dict(key='search', expire=None, type='memory'))

@expose('genshi:smallpress.templates.articles_excerpts')
def excerpts(articles=None, blog=None):
    if articles is None:
        articles=Article.get_published(blog)
    return dict(articles=articles)

@expose('genshi:smallpress.templates.excerpt')
def excerpt(article):
    return article_preview(article)



from tg import request, config

import tg
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Unicode, Integer, DateTime
from sqlalchemy.orm import backref, relation
from sqlalchemy.orm.interfaces import MapperExtension

import os
from datetime import datetime
from tgext.datahelpers.fields import Attachment as DatahelpersAttachment, AttachedFile
from smallpress.model import DeclarativeBase, DBSession
from tgext.pluggable import app_model, primary_key
from tgext.pluggable import call_partial

import mimetypes
mimetypes.init()

try:
    import whoosh
    import whoosh.index
    import whoosh.fields
    WHOOSH_SCHEMA = whoosh.fields.Schema(uid=whoosh.fields.ID(stored=True),
                                         title=whoosh.fields.TEXT(stored=True),
                                         description=whoosh.fields.TEXT,
                                         content=whoosh.fields.TEXT)
    whoosh_enabled = True
except ImportError:
    whoosh_enabled = False

try:
    from sqlalchemy import event
    sqla_events = True
except ImportError:
    sqla_events = False

AttachmentType = tg.config.get('_smallpress',{}).get('attachment_type', AttachedFile)
ENABLE_ARTICLES_CACHE = tg.config.get('_smallpress',{}).get('enable_cache', False)
ARTICLE_CACHE_EXPIRE = tg.config.get('_smallpress',{}).get('cache_expire', 30*60)

class Blog(DeclarativeBase):
    __tablename__ = 'smallpress_blogs'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(100), nullable=False, default=u"Untitled", index=True)

class Article(DeclarativeBase):
    if not sqla_events:
        class Hooks(MapperExtension):
            def before_delete(self, mapper, connection, instance):
                Article.before_delete(mapper, connection, instance)
        __mapper_args__ = {'extension': Hooks()}

    __tablename__ = 'smallpress_articles'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    title = Column(Unicode(150), nullable=False, default=u"Untitled", index=True)
    published = Column(Integer, nullable=False, default=0, index=True)
    private = Column(Integer, nullable=False, default=0)

    author_id = Column(Integer, ForeignKey(primary_key(app_model.User)))
    author = relation(app_model.User, backref=backref('articles'))

    blog_id = Column(Integer, ForeignKey(primary_key(Blog)), nullable=True, index=True)
    blog = relation(Blog, backref=backref('articles'), lazy='joined')

    publish_date = Column(DateTime, nullable=False, default=datetime.now, index=True)
    update_date = Column(DateTime, nullable=False, default=datetime.now)
    description = Column(Unicode(150), nullable=False, default=u'Empty article, edit or delete this')
    content = Column(Unicode(32000), nullable=False, default=u'')

    def refresh_whoosh(self, action=0):
        index_path = config.get('smallpress_whoosh_index', '/tmp/smallpress_whoosh')
        ix = whoosh.index.open_dir(index_path)
        writer = ix.writer()

        if action == 1:
            writer.add_document(uid=unicode(self.uid), title=self.title,
                                content=self.content,
                                description=self.description)
        elif action == -1:
            writer.delete_by_term('uid', unicode(self.uid))
        else:
            writer.update_document(uid=unicode(self.uid), title=self.title,
                                   content=self.content,
                                   description=self.description)
        writer.commit()

    @staticmethod
    def after_update(mapper, connection, obj):
        obj.refresh_whoosh(0)

    @staticmethod
    def after_insert(mapper, connection, obj):
        obj.refresh_whoosh(1)

    @staticmethod
    def whoosh_before_delete(mapper, connection, obj):
        obj.refresh_whoosh(-1)

    @staticmethod
    def before_delete(mapper, connection, obj):
        if whoosh_enabled:
            Article.whoosh_before_delete(mapper, connection, obj)

        DBSession.query(app_model.Tagging).filter(app_model.Tagging.taggable_type == 'Article')\
                                          .filter(app_model.Tagging.taggable_id == obj.uid).delete()

    @staticmethod
    def get_published(blog=None):
        now = datetime.now()
        articles = DBSession.query(Article).filter_by(published=True)\
                                           .filter(Article.publish_date<=now)

        if blog:
            try:
                blog = blog.name
            except:
                pass
            articles = articles.join(Blog).filter(Blog.name==blog)

        articles = articles.order_by(Article.publish_date.desc())
        return articles

    @staticmethod
    def get_all(blog=None):
        articles = DBSession.query(Article)
        if blog:
            try:
                blog = blog.name
            except:
                pass
            articles = articles.join(Blog).filter(Blog.name==blog)
        articles = articles.order_by(Article.publish_date.desc())
        return articles

    @property
    def blog_name(self):
        blog = self.blog
        if blog:
            return blog.name
        else:
            return ''

    def tagging_display(self):
        return call_partial('smallpress.partials:article_preview', article=self)

    def is_owner(self, identity):
        if not identity:
            return False

        return identity['user'] == self.author

    @property
    def caching_options(self):
        if not ENABLE_ARTICLES_CACHE:
            return dict()

        userid = request.identity['user'].user_id if request.identity else 'None'
        try:
            return dict(key='%s-%s-%s-%s' % (userid, 'smallpress-article',
                                             self.uid, self.update_date.strftime('%Y-%m-%d-%H-%M-%S')),
                                             expire=ARTICLE_CACHE_EXPIRE, type="memory")
        except (IndexError, ValueError):
            return dict()

if whoosh_enabled and sqla_events:
    event.listen(Article, 'after_update', Article.after_update)
    event.listen(Article, 'after_insert', Article.after_insert)

if sqla_events:
    event.listen(Article, 'before_delete', Article.before_delete)

class Attachment(DeclarativeBase):
    if not sqla_events:
        class Hooks(MapperExtension):
            def before_delete(self, mapper, connection, instance):
                Attachment.delete_file(mapper, connection, instance)
        __mapper_args__ = {'extension': Hooks()}

    __tablename__ = 'smallpress_attachments'

    uid = Column(Integer, autoincrement=True, primary_key=True)
    name = Column(Unicode(16), nullable=False)
    content = Column(DatahelpersAttachment(AttachmentType))

    article_id = Column(Integer, ForeignKey(Article.uid))
    article = relation(Article, backref=backref('attachments', cascade='all, delete-orphan'))

    @staticmethod
    def delete_file(mapper, connection, obj):
        try:
            os.unlink(obj.content.local_path)
        except:
            pass

    @property
    def mimetype(self):
        return mimetypes.guess_type(self.content.local_path, False)[0]

    @property
    def url(self):
        return self.content.url

if sqla_events:
    event.listen(Attachment, 'before_delete', Attachment.delete_file)



# -*- coding: utf-8 -*-

"""WebHelpers used in tgapp-smallpress."""

from tg import url, request
from tg.i18n import ugettext as _
from webhelpers import date, feedgenerator, html, number, misc, text
from smallpress import model
from tgext.pluggable import app_model

def bold(text):
    return html.literal('<strong>%s</strong>' % text)

def link_buttons(entity, *buttons):
    btns_html = '<div class="smallpress_buttons">'
    for btn in buttons:
        if btn == 'publish' and entity.published:
            continue
        elif btn == 'hide' and not entity.published:
            continue
        elif btn in ('delete', 'rmattachment'):
            btns_html += '''\
    <a href="%(baseurl)s/%(btn)s/%(entity)s" onclick="return confirm('%(confirm_message)s')">
        <img src="/_pluggable/smallpress/buttons/%(btn)s.png"/>
    </a>''' % dict(baseurl=request.controller_state.controller.mount_point,
                   confirm_message=_('Really delete this?'), btn=btn,
                   entity=entity.uid)
        else:
            btns_html += '''\
    <a href="%(baseurl)s/%(btn)s/%(entity)s">
        <img src="/_pluggable/smallpress/buttons/%(btn)s.png"/>
    </a>''' % dict(baseurl=request.controller_state.controller.mount_point, btn=btn, entity=entity.uid)
    btns_html += '</div>'

    return html.literal(btns_html)

def comma_separated_tags(entity):
    return ', '.join((t[0] for t in app_model.Tagging.tag_cloud_for_object(entity)))

def format_published(entity):
    if entity.published:
        return html.builder.HTML.strong('Published')
    else:
        return 'Draft'

def format_date(entity):
    return entity.publish_date.strftime('%Y-%m-%d %H:%M')

def format_link(url):
    return html.builder.HTML.a(url, href=url, target='_blank')

def attachment_preview(attachment):
    attachment_mime_type = attachment.mimetype
    if attachment_mime_type and attachment_mime_type.startswith('image'):
        preview_url = attachment.url
    else:
        preview_url = "/_pluggable/smallpress/images/unknown.png"
    return html.literal('<img src="%s" class="smallpress_attachment_minipreview"/>' % preview_url)


# -*- coding: utf-8 -*-
import sys, os

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

install_requires=[
    "TurboGears2 >= 2.2.0",
    "tgext.pluggable >= 0.1.0",
    "tgext.tagging >= 0.2.1",
    "tgext.datahelpers >= 0.0.9",
    "tgext.ajaxforms",
    "tgext.crud >= 0.4"
]

here = os.path.abspath(os.path.dirname(__file__))
try:
    README = open(os.path.join(here, 'README.rst')).read()
except IOError:
    README = ''

setup(
    name='tgapp-smallpress',
    version='0.6.0',
    description='Pluggable Minimalistic Blog for TurboGears2 with Attachments and Tags',
    long_description=README,
    author='Alessandro Molina',
    author_email='alessandro.molina@axant.it',
    url='http://bitbucket.org/_amol_/tgapp-smallpress',
    keywords='turbogears2.application',
    setup_requires=[],
    packages=find_packages(exclude=['ez_setup']),
    install_requires=install_requires,
    include_package_data=True,
    package_data={'tgapp.smallpress': ['i18n/*/LC_MESSAGES/*.mo',
                                 'templates/*/*',
                                 'public/*/*']},
    entry_points="""
    """,
    dependency_links=[
        ],
    zip_safe=False
)
